/**************************************************************
**                                                           **
** Sistema de Gestion de Pedidos                             **
**                                                           **
** Definicion de objetos y tablas                            **
**                                                           **
**************************************************************/

/******* Declaracion de tipos ***********/

CREATE TYPE item_t;
/

CREATE TYPE pedido_t;
/

CREATE TYPE stock_t;
/

CREATE TYPE lista_tel_t AS VARRAY(10) OF VARCHAR2(20);
/

CREATE TYPE direccion_t AS OBJECT (
   calle     VARCHAR2(100),
   poblacion VARCHAR2(100),
   provincia VARCHAR2(50),
   cod_zip   VARCHAR2(15)
);
/

CREATE TYPE cliente_t AS OBJECT (
   numclient NUMBER,
   nombre    VARCHAR2(200),
   direccion direccion_t,
   telefonos lista_tel_t,
   ORDER MEMBER FUNCTION ord_cliente (x IN cliente_t) RETURN INTEGER,
   PRAGMA RESTRICT_REFERENCES (
     ord_cliente, WNDS, WNPS, RNDS, RNPS)
);
/

CREATE TYPE item_t AS OBJECT (
   numitem   NUMBER,
   stockref  REF stock_t,
   cantidad  NUMBER,
   descuento NUMBER
);
/

CREATE TYPE lista_item_t AS TABLE OF item_t;
/

CREATE TYPE pedido_t AS OBJECT (
   numpedido   NUMBER,
   clientref   REF cliente_t,
   fechapedido DATE,
   fechaenvio  DATE,
   lista_item  lista_item_t,
   dir_envio   direccion_t,
   MAP MEMBER FUNCTION valor RETURN NUMBER,
   PRAGMA RESTRICT_REFERENCES (
     valor, WNDS, WNPS, RNDS, RNPS),
   MEMBER FUNCTION valor_total RETURN NUMBER,
   PRAGMA RESTRICT_REFERENCES (
     valor_total, WNDS, WNPS)
);
/

CREATE TYPE stock_t AS OBJECT (
   numstock NUMBER,
   descr    VARCHAR2(100),
   precio   NUMBER,
   cod_tasa NUMBER
);
/

/******* Declaracion de metodos ***********/

CREATE OR REPLACE TYPE BODY pedido_t AS
   MEMBER FUNCTION valor_total RETURN NUMBER IS
      i      INTEGER;
      stock  stock_t;
      item   item_t;
      total  NUMBER := 0;
      precio NUMBER;
   BEGIN
      FOR i IN 1..SELF.lista_item.COUNT LOOP
          item := SELF.lista_item(i);
          SELECT DEREF(item.stockref) INTO stock FROM DUAL;
          total := total + item.cantidad * stock.precio;
      END LOOP;
      RETURN total;
   END valor_total;
   MAP MEMBER FUNCTION valor RETURN NUMBER IS
   BEGIN
      RETURN numpedido;
   END valor;
END;
/

CREATE OR REPLACE TYPE BODY cliente_t AS
   ORDER MEMBER FUNCTION ord_cliente (
     x IN cliente_t
   ) RETURN INTEGER IS
   BEGIN
      RETURN numclient - x.numclient;
   END ord_cliente;
END;
/

/******* Declaracion de tablas ***********/

CREATE TABLE clientes_tab OF cliente_t (
   numclient PRIMARY KEY,
   nombre NOT NULL
);

CREATE TABLE stock_tab OF stock_t (
   numstock PRIMARY KEY,
   descr NOT NULL,
   precio NOT NULL,
   cod_tasa NOT NULL
);

CREATE TABLE pedidos_tab OF pedido_t (
   PRIMARY KEY (numpedido),
   SCOPE FOR (clientref) IS clientes_tab
)
NESTED TABLE lista_item STORE AS lista_item_tab;

ALTER TABLE lista_item_tab
   ADD (SCOPE FOR (stockref) IS stock_tab);

CREATE INDEX lista_item_index
   ON lista_item_tab (numitem, NESTED_TABLE_ID);

-- Algunas secuencias utiles
CREATE SEQUENCE cliente_seq;
CREATE SEQUENCE item_seq;
CREATE SEQUENCE pedido_seq;
CREATE SEQUENCE stock_seq;
